// hub_mem

/*
-------------------------------------------------------------------------------
Copyright 2014 Parallax Inc.

This file is part of the hardware description for the Propeller 1 Design.

The Propeller 1 Design is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by the
Free Software Foundation, either version 3 of the License, or (at your option)
any later version.

The Propeller 1 Design is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
more details.

You should have received a copy of the GNU General Public License along with
the Propeller 1 Design.  If not, see <http://www.gnu.org/licenses/>.
-------------------------------------------------------------------------------
*/
//
// Magnus Karlsson 20140818     RAM is now 64KB
//
// RR20140816   ROM to use new unscrambled code and preset with $readmemh
//              ROM is now 4KB ($F000..$FFFF) and preset with interpreter/booter/runner
//              RAM may be expanded to fill available space up to 60KB
// RR20140816   Bigger Hub RAM & No ROM
//              RAM is 48KB and remaps 48-64KB to 32-48KB

module              hub_mem
(
input               clk_cog,
input               ena_bus,

input               w,
input        [3:0]  wb,
input       [13:0]  a,
input       [31:0]  d,

output      [31:0]  q
);


// 16KB x 32 (64KB) ram with byte-write enables ($0000..$FFFF)
reg [7:0] ram3 [16*1024-1:0];   // 4 x 16KB
reg [7:0] ram2 [16*1024-1:0];
reg [7:0] ram1 [16*1024-1:0];
reg [7:0] ram0 [16*1024-1:0];

// pre-load ROM
initial
begin
    //Upper part of ROM (booter, interpreter, copyright, runner) @ 0xF004/4
    $readmemh ("ROM_$F004-$FFFF_BYTE_0.hex", ram0, 15*1024+1);
    $readmemh ("ROM_$F004-$FFFF_BYTE_1.hex", ram1, 15*1024+1);
    $readmemh ("ROM_$F004-$FFFF_BYTE_2.hex", ram2, 15*1024+1);
    $readmemh ("ROM_$F004-$FFFF_BYTE_3.hex", ram3, 15*1024+1);
    
    //Font @ 0x8000/4
    $readmemh ("rom_8000_bfff_font_0.hex", ram0, 8192);
    $readmemh ("rom_8000_bfff_font_1.hex", ram1, 8192);
    $readmemh ("rom_8000_bfff_font_2.hex", ram2, 8192);
    $readmemh ("rom_8000_bfff_font_3.hex", ram3, 8192);
    
    //Log table @ 0x8000/4
    $readmemh ("rom_c000_cfff_log_0.hex", ram0, 12288);
    $readmemh ("rom_c000_cfff_log_1.hex", ram1, 12288);
    $readmemh ("rom_c000_cfff_log_2.hex", ram2, 12288);
    $readmemh ("rom_c000_cfff_log_3.hex", ram3, 12288);
    
    //Anti-log table @ 0xD000/4
    $readmemh ("rom_d000_dfff_antilog_0.hex", ram0, 13312);
    $readmemh ("rom_d000_dfff_antilog_1.hex", ram1, 13312);
    $readmemh ("rom_d000_dfff_antilog_2.hex", ram2, 13312);
    $readmemh ("rom_d000_dfff_antilog_3.hex", ram3, 13312);
    
    //Sine table @ 0xE000/4
    $readmemh ("rom_e000_f003_sine_0.hex", ram0, 14336);
    $readmemh ("rom_e000_f003_sine_1.hex", ram1, 14336);
    $readmemh ("rom_e000_f003_sine_2.hex", ram2, 14336);
    $readmemh ("rom_e000_f003_sine_3.hex", ram3, 14336);
    
end

reg [7:0] ram_q3;
reg [7:0] ram_q2;
reg [7:0] ram_q1;
reg [7:0] ram_q0;


always @(posedge clk_cog)
begin
    if (ena_bus && w && wb[3])
        ram3[a[13:0]] <= d[31:24];
    if (ena_bus)
        ram_q3 <= ram3[a[13:0]];
end

always @(posedge clk_cog)
begin
    if (ena_bus && w && wb[2])
        ram2[a[13:0]] <= d[23:16];
    if (ena_bus)
        ram_q2 <= ram2[a[13:0]];
end

always @(posedge clk_cog)
begin
    if (ena_bus && w && wb[1])
        ram1[a[13:0]] <= d[15:8];
    if (ena_bus)
        ram_q1 <= ram1[a[13:0]];
end

always @(posedge clk_cog)
begin
    if (ena_bus && w && wb[0])
        ram0[a[13:0]] <= d[7:0];
    if (ena_bus)
        ram_q0 <= ram0[a[13:0]];
end

assign q        = {ram_q3, ram_q2, ram_q1, ram_q0};
endmodule
